export declare function getSharedId(frameId: string, documentId: string, backendNodeId: number): string;
export declare function parseSharedId(sharedId: string): {
    frameId: string | undefined;
    documentId: string;
    backendNodeId: number;
} | null;
