/**
 * @license
 * Copyright 2017 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
export * from 'puppeteer-core';
import { PuppeteerNode } from 'puppeteer-core';
import { getConfiguration } from './getConfiguration.js';
/**
 * @public
 */
// @ts-expect-error using internal API.
const puppeteer = new PuppeteerNode({
    isPuppeteerCore: false,
    configuration: getConfiguration,
});
export const { 
/**
 * @public
 */
connect, 
/**
 * @public
 */
defaultArgs, 
/**
 * @public
 */
executablePath, 
/**
 * @public
 */
launch, 
/**
 * @public
 */
trimCache, 
/**
 * @public
 */
setFollowSymlinks, } = puppeteer;
export default puppeteer;
//# sourceMappingURL=puppeteer.js.map