import type { Session } from '../protocol/generated/webdriver-bidi.js';
export interface MapperOptions {
    acceptInsecureCerts?: boolean;
    unhandledPromptBehavior?: Session.UserPromptHandler;
    'goog:prerenderingDisabled'?: boolean;
    'goog:disableNetworkDurableMessages'?: true;
}
