/**
 * Copyright 2026 Google LLC.
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * THIS FILE IS AUTOGENERATED by cddlconv 0.1.7.
 * Run `node tools/generate-bidi-types.mjs` to regenerate.
 * @see https://github.com/w3c/webdriver-bidi/blob/master/index.bs
 */
import z from 'zod';
export declare namespace DigitalCredentials {
    const VirtualWalletActionSchema: z.ZodLazy<z.ZodEnum<["decline", "respond", "wait", "clear"]>>;
}
export declare namespace DigitalCredentials {
    const SetVirtualWalletBehaviorParametersSchema: z.ZodLazy<z.ZodObject<{
        action: z.ZodLazy<z.ZodEnum<["decline", "respond", "wait", "clear"]>>;
        context: z.ZodOptional<z.ZodString>;
        protocol: z.ZodOptional<z.ZodString>;
        response: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodAny>>;
    }, "strip", z.ZodTypeAny, {
        action: "clear" | "respond" | "decline" | "wait";
        context?: string | undefined;
        response?: Record<string, any> | undefined;
        protocol?: string | undefined;
    }, {
        action: "clear" | "respond" | "decline" | "wait";
        context?: string | undefined;
        response?: Record<string, any> | undefined;
        protocol?: string | undefined;
    }>>;
}
export declare namespace DigitalCredentials {
    const SetVirtualWalletBehaviorSchema: z.ZodLazy<z.ZodObject<{
        method: z.ZodLiteral<"digitalCredentials.setVirtualWalletBehavior">;
        params: z.ZodLazy<z.ZodObject<{
            action: z.ZodLazy<z.ZodEnum<["decline", "respond", "wait", "clear"]>>;
            context: z.ZodOptional<z.ZodString>;
            protocol: z.ZodOptional<z.ZodString>;
            response: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodAny>>;
        }, "strip", z.ZodTypeAny, {
            action: "clear" | "respond" | "decline" | "wait";
            context?: string | undefined;
            response?: Record<string, any> | undefined;
            protocol?: string | undefined;
        }, {
            action: "clear" | "respond" | "decline" | "wait";
            context?: string | undefined;
            response?: Record<string, any> | undefined;
            protocol?: string | undefined;
        }>>;
    }, "strip", z.ZodTypeAny, {
        params: {
            action: "clear" | "respond" | "decline" | "wait";
            context?: string | undefined;
            response?: Record<string, any> | undefined;
            protocol?: string | undefined;
        };
        method: "digitalCredentials.setVirtualWalletBehavior";
    }, {
        params: {
            action: "clear" | "respond" | "decline" | "wait";
            context?: string | undefined;
            response?: Record<string, any> | undefined;
            protocol?: string | undefined;
        };
        method: "digitalCredentials.setVirtualWalletBehavior";
    }>>;
}
export declare namespace DigitalCredentials {
    const SetVirtualWalletBehaviorResultSchema: z.ZodLazy<z.ZodLazy<z.ZodLazy<z.ZodRecord<z.ZodString, z.ZodAny>>>>;
}
